from typing import Any

class PooledObject:
    create_time: float
    last_borrow_time: float
    keeped_object: Any
    def __init__(self, obj: Any) -> None: ...
    def use(self) -> Any: ...
    def update_brrow_time(self) -> PooledObject: ...
